/* ALIS ********* Advanced LIST service for IRC Networks *****************\
*                                                                         *
* Copyright (c) 1999 Thomas 'Engerim' Kuiper                              *
* Comments to: engerim@dachbu.de                                          *
*                                                                         *
* This program is distributed in the hope that it will be useful,         *
* but WITHOUT ANY WARRANTY; without even the implied warranty of          *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE and stuff.          *
* And its totally free to copy and change too.                            *
*                                                                         *
* Some code from tkserv.c and other well known open source programs       *
* like fetchmail for the nice socketthingies.                             *
**************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <limits.h>
#include <signal.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include "servdef.h"
#include "config.h"
#include "alis.h"
#include "list.h"
#include "hash.h"
#include "mem.h"
#include "io.h"
#include "misc.h"

#define SERVICE_VERSION "2.2"
#define SERVICE_MASK1 SERVICE_WANT_VCHANNEL + SERVICE_WANT_MODE + SERVICE_WANT_TOPIC
#define SERVICE_MASK2 SERVICE_WANT_VCHANNEL + SERVICE_WANT_MODE + SERVICE_WANT_TOPIC

#define CTRLBOLD 0x2

#define AUTHORS "Thomas Kuiper & Michal Svoboda"

static int sockserv; /* server socket */
struct chaninfo *chanlist[CHANNEL_HASH_SIZE]; /* channels */
int inburst = 1;
int totalchannels = 0;

int qmatch (char *ptr1,char *ptr2);
char *strlwr(char *str);
char *strupr(char *str);
void init_signals();
static void onhup(int s);
int mycmp (char *str1,char *str2);
void die(char *);

/* yet silly hashing */
int channel_hash_func(char *name, int id)
{
    unsigned int r = 0;
    while (*name)
    {
	r <<= 3;
	r += (unsigned char)*name++;
    }

    return r % CHANNEL_HASH_SIZE;
}

int squeries = 0, listqueries = 0;
int modeok(char *m)
{
	char *p;

	for (p = m; *p; p++)
		switch (*p) 
		{
			case 'p':
			case 's':
				return 0;
		}
	return 1;
}

void communicate()
{
    int rcvd,i,isnew,m,hashused,hashbox,hashmax,hashmin;
    int replied,opttype;
    int match, done;
    struct chaninfo *tmp = NULL;
    char cbuf[513];
    char *param[15];
    char *token;
#ifdef LOGSQUERY
    FILE *fplog;
#endif
    char seps[] = " ";

    rcvd = 0;
    init_signals();
    while(rcvd != -1)
    {
    rcvd = SockRead(sockserv, cbuf, sizeof(cbuf));
    AddDebug(cbuf);

    param[0] = cbuf;
    done = 0;
    for (token = cbuf, i = 0; token && i < 15; token++)
    {
	if (*token == '\n' || *token == '\r')
	{
	    *token = 0;
	    break;
	}
	if (!done && *token == ' ')
	{
	    *token = 0;
	    i++;
	    param[i] = token + 1;
	    if (*param[i] == ':')
	    {
		param[i]++;
		done = 1;
	    }
	}
    }

    for (i++; i < 15; i++)
	param[i] = NULL;

    if (param[1] && *param[1] == '2') inburst = 0;
   
    if (strcmp(param[0],"CHANNEL") == 0) /* new channel or user number change*/
    {
       
       /* so we got a new channel, first lets check if he is really new */
       
       isnew = 1;
	strlwr(param[1]);
	if (inburst == 0)
        {
	    tmp = (struct chaninfo *)find_in_hash_by_name((Hash)chanlist, 
		    param[1], 0, NULL, channel_hash_func);
	    if (tmp) /* known channel */
	    {
		tmp->members = atoi(param[2]);
		if (!tmp->members)
		{
		    totalchannels--;
		    remove_from_hash((Hash)chanlist, (List *)tmp, 
			    channel_hash_func);
		}
		isnew = 0;
	    }
        }

	if (isnew)
	{
	    totalchannels++;

	    tmp = (struct chaninfo *)add_to_hash((Hash)chanlist, sizeof(struct chaninfo), param[1], 0, channel_hash_func);
	    
	    strcpy(tmp->topicnick,"*unknown*");	    
	    strcpy(tmp->mode,"+");	    
	    strcpy(tmp->topic,"");
	    tmp->members = atoi (param[2]);
	}

	continue;
    }

    if (strcmp(param[0],"MODE") == 0)
    {
	strlwr(param[1]);
	
        if (!inburst || strcmp(tmp->name, param[1]))
        {
	    tmp = (struct chaninfo *)find_in_hash_by_name((Hash)chanlist, 
		    param[1], 0, NULL, channel_hash_func);
        }
        
        if (tmp) strcpy(tmp->mode, param[2]);
	continue;
    }
    
    if (strcmp(param[0],"TOPIC") == 0) 
    {
	strlwr(param[1]);
	
        if (!inburst || strcmp(tmp->name, param[1]))
        {
	    tmp = (struct chaninfo *)find_in_hash_by_name((Hash)chanlist, 
		    param[1], 0, NULL, channel_hash_func);
        }
	
	if (tmp)
	{
            strcpy(tmp->topic, param[2]);

            strcpy(tmp->topicnick, "*unknown*");
	}


	continue;
    }

    if (strcmp(param[0],"PING") == 0)
    {
	SockPrintf(sockserv,"PONG %s\n",param[1]);
	continue;
    }


    if (param[1] && strcmp(param[1],"TOPIC") == 0) /* user sets a topic */
    {
	strlwr(param[2]);
	
	tmp = (struct chaninfo *)find_in_hash_by_name((Hash)chanlist, 
		    param[2], 0, NULL, channel_hash_func);
	
	if (tmp)
	{
	    strcpy(tmp->topic, param[3]);
	    strcpy(tmp->topicnick, (param[0])+1);
	}
	continue;
    }
    
        
    if (param[1] && strcmp(param[1],"SQUERY") == 0) /* someone talks with me */
    {

#ifdef LOGSQUERY
fplog = fopen ("squery.log","a");
for (i = 0; i < 15; i++)
  fprintf(fplog,"%s ",param[i]?param[i]:"");
fprintf(fplog,"\n");
fclose(fplog);
#endif
        strlwr(param[3]);
	param[4] = strchr(param[3], ' ');
	if (param[4])
	{
	    *param[4] = 0;
	    param[4]++;
            strlwr(param[4]);
	}

	param[0]++;
	
	squeries++;

	if (strcmp(param[3],"admin") == 0)
	{
	    SockPrintf(sockserv,"NOTICE %s :Administrative info about %s:\n",
	        param[0],SERVICE_NAME);
	    SockPrintf(sockserv,"NOTICE %s :%s <%s>\n",
	        param[0],ADMIN_NAME,ADMIN_EMAIL);
	    continue;
	
	}

	if (strcmp(param[3],"die") == 0)
	{
#ifdef SERVICE_CTRLPASS
	    if (param[4] && !strcmp(param[4], SERVICE_CTRLPASS))
	    {
		char *tmp;
	        SockPrintf(sockserv,"NOTICE %s :*poof*\n", param[0]);
		tmp = my_calloc(1, 600);
		sprintf(tmp, "destroyed by %s. good-bye Mr. Anderson!\n", param[0]);
		die(tmp);
	    }
	    else
		SockPrintf(sockserv,"NOTICE %s :you fool!\n", param[0]);
#else
	    SockPrintf(sockserv,"NOTICE %s :... but i don't want to!\n", param[0]);
#endif
	    continue;
	}
	
	if (strcmp(param[3],"help") == 0)
	{
	    
	    if (!param[4] || !(*param[4]))
	    {
		SockPrintf(sockserv,"NOTICE %s :Alis help index\n",param[0]);
		SockPrintf(sockserv,"NOTICE %s :Use /SQUERY %s HELP <topic>\n",param[0], SERVICE_NAME);
		SockPrintf(sockserv,"NOTICE %s :Available topics: ADMIN, DIE, HASH, INFO, LIST, STATUS, VERSION\n",param[0]);
		SockPrintf(sockserv,"NOTICE %s :For LIST examples use /SQUERY %s HELP EXAMPLES\n",param[0],SERVICE_NAME);
		continue;
	    }
	    else
	    {	
		if (strcmp(param[4],"admin") == 0)
		{
		    SockPrintf(sockserv,"NOTICE %s :Shows administrative information\n",param[0]);
		    continue;
		}
		if (strcmp(param[4],"die") == 0)
		{
		    SockPrintf(sockserv,"NOTICE %s :don't be silly :P\n",param[0]);
		    continue;
		}
		if (strcmp(param[4],"hash") == 0)
		{
		    SockPrintf(sockserv,"NOTICE %s :shows hash usage (only to The One)\n",param[0]);
		    continue;
		}
		if (strcmp(param[4],"info") == 0)
		{
		    SockPrintf(sockserv,"NOTICE %s :Shows information about this software\n",param[0]);
		    continue;
		}
		if (strcmp(param[4],"status") == 0)
		{
		    SockPrintf(sockserv,"NOTICE %s :Shows status information about this software\n",param[0]);
		    continue;
		}
		if (strcmp(param[4],"version") == 0)
		{
		    SockPrintf(sockserv,"NOTICE %s :Shows current version\n",param[0]);
		    continue;
		}
		if (strcmp(param[4],"list") == 0)
		{
		    SockPrintf(sockserv,"NOTICE %s :Searches for a channel\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :/SQUERY %s mask [-options]\n",param[0], SERVICE_NAME);
		    SockPrintf(sockserv,"NOTICE %s :  mask               channels to search for, * and ? wildcards accepted\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :  options:\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :   -min x            minimum users on a channel\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :   -max x            maximum users on a channel\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :   -m mode           check against channel mode\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :    +tn means channel has tn set\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :    -im means channel has im unset\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :    =tn means channel is exactly +tn nothing more and nothing less\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :   -mode mode        same as above\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :   -t string         topic of the channel shall contain string\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :   -topic string     same as above\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :   -s [m][t]         show modes (m) and who set the topic (t)\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :   -show [m][t]      same as above\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :See also: HELP EXAMPLES\n",param[0]);
		    continue;
		}
		if (strcmp(param[4],"examples") == 0)
		{
		    SockPrintf(sockserv,"NOTICE %s :LIST Examples:\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :/SQUERY %s #foo* -min 10 -mode =tn\n",param[0], SERVICE_NAME);

		    SockPrintf(sockserv,"NOTICE %s :Lists all channels who contain foo (#foo.int,#foo,...)\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :if the mode is exactly = tn.\n",param[0]);

		    SockPrintf(sockserv,"NOTICE %s :/SQUERY %s * -min 10 -t http\n",param[0], SERVICE_NAME);

		    SockPrintf(sockserv,"NOTICE %s :Lists all channels who contain http as word in the topic.\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :Examples: \"http://www.foo.org\",\"http is cool\",\"my httpd crashed\"\n",param[0]);

		    SockPrintf(sockserv,"NOTICE %s :/SQUERY %s * -min 10 -t http -show mt\n",param[0], SERVICE_NAME);

		    SockPrintf(sockserv,"NOTICE %s :Like above, but it will show the modes of the channel\n",param[0]);
		    SockPrintf(sockserv,"NOTICE %s :and the nick who set the topic.\n",param[0]);
		    continue;
		}
		SockPrintf(sockserv,"NOTICE %s :No such help topic: \"%s\". Use /SQUERY HELP\n",param[0],param[4]);

	    }
	    continue;
	}
	
	if (strcmp(param[3],"status") == 0)
	{
	    SockPrintf(sockserv,"NOTICE %s :Alis %s status:\n",param[0],SERVICE_VERSION);
	    SockPrintf(sockserv,"NOTICE %s :totalchannels: %i\n",
	        param[0],totalchannels); 
	    SockPrintf(sockserv,"NOTICE %s :SQUERY statistic: %i total requests, %i list requests\n",
	        param[0],squeries,listqueries);
	    continue;
	}
	if (strcmp(param[3],"hash") == 0)
	{
#ifdef SERVICE_CTRLPASS
	    if (param[4] && !strcmp(param[4], SERVICE_CTRLPASS))
	    {
	    SockPrintf(sockserv,"NOTICE %s :Alis %s hash usage:\n",param[0],SERVICE_VERSION);
	    SockPrintf(sockserv,"NOTICE %s :totalchannels: %i\n",
	        param[0],totalchannels);
	    SockPrintf(sockserv,"NOTICE %s :hash size: %i\n",
		param[0],CHANNEL_HASH_SIZE);    
	    hashused = 0;
	    hashmin = INT_MAX;
	    hashmax = 0;
	    for (i = 0; i < CHANNEL_HASH_SIZE; i++)
	    {
		if (chanlist[i])
		    hashused++;
		for (tmp = chanlist[i], hashbox = 0; tmp; tmp = tmp->next, hashbox++);
		if (hashbox && hashbox < hashmin)
		    hashmin = hashbox;
		if (hashbox > hashmax)
		    hashmax = hashbox;
	    }
	    SockPrintf(sockserv, "NOTICE %s :%i boxes used\n",
		    param[0], hashused);
	    SockPrintf(sockserv, "NOTICE %s :smallest box contains %i items\n",
		    param[0], hashmin);
	    SockPrintf(sockserv, "NOTICE %s :largest box contains %i items\n",
		    param[0], hashmax);
	    SockPrintf(sockserv, "NOTICE %s :average box usage is %f items\n",
		    param[0], (double) totalchannels / (double) hashused);
	    SockPrintf(sockserv, "NOTICE %s :optimal box usage should be %f items\n",
		    param[0], (double) totalchannels / (double) CHANNEL_HASH_SIZE);
	    }
	    else
	    SockPrintf(sockserv, "NOTICE %s :you're not The One so don't play around!\n",
	            param[0]);
	    
#else
	    SockPrintf(sockserv, "NOTICE %s :they disabled this command! those bastards!", param[0]);
#endif
	    continue;
	}
	if (strcmp(param[3],"info") == 0)
	{
	    SockPrintf(sockserv,"NOTICE %s :Information about %s:\n",
	        param[0],SERVICE_NAME);
	    SockPrintf(sockserv,"NOTICE %s :Programmed by %s\n",
	        param[0], AUTHORS);
	    SockPrintf(sockserv,"NOTICE %s :For more information mail to %s\n",
	        param[0], ADMIN_EMAIL);
		
	    continue;
	
	}
	if (strcmp(param[3],"version") == 0)
	{
	    SockPrintf(sockserv,"NOTICE %s :%s version %s\n",
	        param[0],SERVICE_NAME,SERVICE_VERSION);
	    continue;
	}
	
	if (strcmp(param[3],"list") == 0)
	{
	    int bad = 0, allmatch = 0;
	    listqueries++;
	    *qmode = 0;	
    	    *qtopic = 0;
	    *qchannel = 0;
    	    
	    qmodetype = 0;
	    qminusers = 0;
	    shownick  = 0;
	    showmode  = 0;
	    qmaxusers = INT_MAX;
	    
	    opttype = 0;
	    
	    /* parse fquery */
	    if (param[4] && (*param[4]))
                for (token = strtok( param[4], seps ); token; token = strtok( NULL, seps ))
            {
		    if (opttype == 1) /* min users */
		    {
			qminusers = atoi(token);
		        opttype = 0;
			continue;
		    }
		    if (opttype == 2) /* max users */
		    {
			qmaxusers = atoi(token);
		        opttype = 0;
			continue;
		    }
		    if (opttype == 3) /* mode */
		    {
			if (token[0] == '=') qmodetype = 1;
			if (token[0] == '+') qmodetype = 2;
			if (token[0] == '-') qmodetype = 3;
			if (!qmodetype)
			{
	                    SockPrintf(sockserv,"NOTICE %s :Invalid mode type: %c\n",param[0],token[0]);
			    bad++;
			}
			strcpy(qmode,token);
		        opttype = 0;
			continue;

		    }
		    if (opttype == 4) /* topic */
		    {
			strcpy(qtopic,token);
		        opttype = 0;
			continue;
		    }
		    if (opttype == 5) /* show */
		    {
			char *p;
			for (p = token; *p; p++)
			{
			    switch (*p)
			    {
				case 'm':
				    showmode = 1;
				    break;
			        case 't':
				    shownick = 1;
				    break;
				default:
				    SockPrintf(sockserv,"NOTICE %s :Invalid -SHOW option: %c\n",param[0],*p);
				    bad++;
			    }
			}
		        opttype = 0;
			continue;
		    }

		    
		    if (strcmp(token,"-min") == 0) opttype = 1;
		    if (strcmp(token,"-max") == 0) opttype = 2;
		    if (strcmp(token,"-mode") == 0) opttype = 3;
		    if (strcmp(token,"-m") == 0) opttype = 3;
		    if (strcmp(token,"-t") == 0) opttype = 4;
		    if (strcmp(token,"-topic") == 0) opttype = 4;
		    if (strcmp(token,"-show") == 0) opttype = 5;
		    if (strcmp(token,"-s") == 0) opttype = 5;
		    if (!opttype)
		    {
			if (*token == '-')
			{
			    SockPrintf(sockserv,"NOTICE %s :Invalid option: %s\n",param[0],token);
			    bad++;
			}
			else
			{
			    if (*qchannel)
			    {
			        SockPrintf(sockserv,"NOTICE %s :Garbage word: %s\n",param[0],token);
				bad++;
			    }
			    else
		                strcpy(qchannel,token);
			}
		    }
		

	    }

	    if (!*qchannel)
	    {
	        SockPrintf(sockserv,"NOTICE %s :You didn't specify a channel mask.\n",param[0]);
		bad++;
	    }

	    if (bad)
	    {
		SockPrintf(sockserv,"NOTICE %s :Your query contains %i errors. Use /SQUERY HELP LIST\n",param[0], bad);
		continue;
	    }

	    
	    SockPrintf(sockserv,"NOTICE %s :Query summary: searching for channels matching \"%s\"", param[0], qchannel);
	    if (qminusers)
		SockPrintf(sockserv,", min %i users",qminusers);
	    if (qmaxusers < INT_MAX)
		SockPrintf(sockserv,", max %i users",qmaxusers);
	    if (qmodetype)
	    {
		SockPrintf(sockserv,", mode ");
		switch (qmodetype)
		{
		    case 1:
			SockPrintf(sockserv,"=");
			break;
		    case 2:
			SockPrintf(sockserv,"+");
			break;
		    case 3:
			SockPrintf(sockserv,"-");
			break;
		}
		SockPrintf(sockserv,"%s",qmode);
	    }
	    if (*qtopic)
		SockPrintf(sockserv,", topic containing \"%s\"", qtopic);
	    
	    if (showmode)
		SockPrintf(sockserv,", showing mode");

	    if (shownick)
		SockPrintf(sockserv,", showing who set topic");

	    SockPrintf(sockserv, ".\n");

	    SockPrintf(sockserv,"NOTICE %s :Returning a maximum of %i channel names.\n",param[0],MAXREPLY);
	
	    replied = 0;
	    match = 0;

	    if (!strcmp(qchannel, "*"))
		allmatch = 1;
	
	    for (i = 0;i < CHANNEL_HASH_SIZE;i++)
		for (tmp = chanlist[i]; tmp; tmp = tmp->next)
	    {
	        if (tmp->members && modeok(tmp->mode))
		{
		    if (allmatch || qmatch(tmp->name,qchannel))
		    {   
			match = 1;

			if (tmp->members < qminusers) match = 0;
			if (tmp->members > qmaxusers) match = 0;
			
			switch (qmodetype)
			{
			case 1:
			    if (strcmp(tmp->mode,qmode)) match = 0;
			    break;
			case 2:
			    for (m = 0;m<strlen(qmode);m++)
			    if (!strchr(tmp->mode,qmode[m])) match = 0;
			    break;
			case 3:
			    for (m = 1;m<strlen(qmode);m++)
			    {
				if (strchr(tmp->mode,qmode[m])) match = 0;
			    }
			    break;
			}
			
			if (!strstr(tmp->topic,qtopic)) match = 0;
		
			if (match)
			{
			    replied++;
			    if (replied < MAXREPLY)
			    {
			    
				SockPrintf(sockserv,"NOTICE %s :",param[0]);
				SockPrintf(sockserv,"%-50s",tmp->name);
				if (showmode == 1)
				    SockPrintf(sockserv," %-12s",tmp->mode);
				SockPrintf(sockserv," %c%3i%c:",CTRLBOLD,tmp->members,CTRLBOLD);
				SockPrintf(sockserv," %s",tmp->topic);
				if (shownick == 1)
				    SockPrintf(sockserv," (%s)",tmp->topicnick);

				SockPrintf(sockserv,"\n");
			    }
			}
		    }
		}
	    }

	    SockPrintf(sockserv,"NOTICE %s :found %i visible channels.\n",
	    param[0],replied);
	    continue;
	    
	}
	SockPrintf(sockserv,"NOTICE %s :Unrecognized command: \"%s\". Use /SQUERY HELP\n",param[0], param[3]);
    }
}

/* hm I recommed the album Karma from Delerium
its a great ambient album, which features Sarah McLaughlin.*/

   close(sockserv);
}

void init_signals()
{
    (void)signal(SIGHUP, onhup);
}

void onhup(int s)
{
    FILE *fp;
    int i;
    struct chaninfo *tmp;
    
    if (s == SIGHUP)
    {
	fp = fopen ("channels.txt","w");
	for (i = 0;i < CHANNEL_HASH_SIZE;i++)
	    for (tmp = chanlist[i]; tmp; tmp = tmp->next)
	{
	
		fprintf(fp,"%s (%s) %i - %s (%s)\n",tmp->name,tmp->mode,
		tmp->members,tmp->topic,tmp->topicnick);
	}
	fclose(fp);
    }				    					

    signal(s, onhup);
}

void die(char *s)
{
#ifdef SDEBUG
    puts(s);
#endif
    SockPrintf(sockserv, "QUIT :%s", s);
    unlink(PID_FILE);
    sleep(10);
    exit(0);
}

void connect_me()
{
    sockserv = SockOpen(SERVER_NAME,SERVER_PORT);

    printf("Starting Service...\n");
    if (sockserv < 0)
    {
    printf ("Error connecting to IRC Server: %s\n", strerror(-sockserv));
    exit(1);
    }
    printf("Sending password...\n");
    
    SockPrintf(sockserv,"PASS %s\n", SERVICE_PASS);
    printf("Introducing...\n");
    SockPrintf(sockserv,"SERVICE %s %s %s %i 0 :%s\n",
    SERVICE_NAME,SERVICE_HOST,SERVICE_DIST,SERVICE_MASK2,SERVICE_DESC);
    
    SockPrintf(sockserv,"SERVSET %i %i\n",SERVICE_MASK1,SERVICE_MASK2);

    SockPrintf(sockserv,"LUSERS\n");
}

int main(int argc, char *argv[])
{
    FILE *fp;
    
    init_signals();


#ifndef SDEBUG

    switch (fork())
        {
            case -1:
                perror("fork()");
                exit(3);
            case 0:
                close(STDIN_FILENO);
                close(STDOUT_FILENO);
                close(STDERR_FILENO);
	        if (setsid() == -1)
                    exit(4);
            break;
            default:
            printf("daemonized.\n");
            return 0;
        }
#endif

    fp = fopen(PID_FILE,"w");
    if (fp != NULL)
    {
    fprintf (fp,"%i\n",getpid());
    fclose(fp);
    }
    memset(chanlist, 0, sizeof(chanlist));

    for (;;)
    {
	connect_me();
        communicate();
	sleep(RECONNECT_DELAY);
    }
    
    return 0;
}


